--- CelebrationOverlay

class("CelebrationOverlay")

require("sphinx.SphinxSound")
require("sphinx.NiceJobDecal")
require("sphinx.Confetti")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_INDEX = {
  NICE_JOB = 0,
  CONFETTI = 10
}

local DECAL_WIDTH_FRACTION = 0.5
local DECAL_HEIGHT_FRACTION = 0.1
local CONFETTI_HEIGHT_FRACTION = 0.8
local CONFETTI_EMIT_HEIGHT = 0.3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CelebrationOverlay:getCelebrationAnimation()
  local animation = AHSpawn.new()
  animation:addAction(self.niceJobDecal:getAnimation())
  animation:addAction(self.confetti:getAnimation())
  animation:addAction(function() SphinxSound.playGameVictory() end)

  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildOverlay(self)
  local niceJobDecal = NiceJobDecal.new(self.roundNumber)
  self:insertAtLocationFractional(niceJobDecal,
                                  DECAL_WIDTH_FRACTION,
                                  DECAL_HEIGHT_FRACTION,
                                  Z_INDEX.NICE_JOB)

  local confetti = Confetti.new(self:getWidth(), self:getHeightPart(CONFETTI_HEIGHT_FRACTION), CONFETTI_EMIT_HEIGHT)
  self:insertAtCenter(confetti, Z_INDEX.CONFETTI)

  self.niceJobDecal = niceJobDecal
  self.confetti = confetti
end

-- Constructor ------------------------

function CelebrationOverlay.new(roundNumber)
  local self = Group.newFullScreenSafe()
  Mixin.mix(self, CelebrationOverlay)

  self.roundNumber = roundNumber

  buildOverlay(self)

  return self
end
