--- ClueDisplayCarousel

class("ClueDisplayCarousel")

require("sphinx.SphinxCarousel")
require("sphinx.SphinxColors")
require("utils.TextUtils")
require("Event")
require("text.SizeConstrainedTextArea")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_INDEX = {
  CARD_BASE = 0,
  CARD_TEXT = 10
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function makeCard(self, text)
  local card = Group.new(self:getWidth(), self:getHeight())
  local cornerRadius = self:getWidth() / 25
  local color = SphinxColors.getRoundColor("dark")
  local base = FilledRoundedRectangle.new(self:getWidthPart(0.95), self:getHeight(), cornerRadius, color)
  card:insertAtCenter(base, Z_INDEX.CARD_BASE)
  base:setOpacity(0)
  card.base = base

  local baseAttributes = {
    font = Font.new(FontFaces.BOLD, 18)
  }
  local attributesForLabels = {
    blue = {color = {0.22, 0.68, 0.82}}
  }

  local lineSpacing = TextUtils.getLineSpacing(18, 26)
  local textBox = SizeConstrainedTextArea.new(
    text,
    ReferenceDevice.getScaledWidth(294),
    card:getHeight(),
    baseAttributes,
    attributesForLabels,
    10,
    Alignment.CENTER,
    nil,
    lineSpacing)
  card:insertAtCenter(textBox, Z_INDEX.CARD_TEXT)
  textBox:setOpacity(0)
  card.text = textBox

  return card
end

local function getOnScrollEndedCallback(self)
  return function()
    local currentStep = 1 + self.carousel.progressIndicator:getProgressStep()
    if currentStep ~= self.currentStep and (self.scrollTargetStep == nil or currentStep == self.scrollTargetStep) then
      self.currentStep = currentStep
      self.stepChanged:call(currentStep)
      self.scrollTargetStep = nil
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ClueDisplayCarousel:setTouchEnabled(enabled)
  self.carousel:setTouchEnabled(enabled)
end

function ClueDisplayCarousel:getCurrentStep()
  return self.currentStep
end

function ClueDisplayCarousel:isManualScroll()
  return self.scrollTargetStep == nil
end

function ClueDisplayCarousel:scrollToCard(cardIndex)
  self.scrollTargetStep = cardIndex
  self.carousel:scrollToCard(cardIndex)
end

function ClueDisplayCarousel:completeCard(cardIndex, nextCardIndex)
  local card = self.cards[cardIndex]
  local duration = 0.5
  local ease = nil
  local color = SphinxColors.getRoundColor("light")

  if nextCardIndex ~= nil and self.currentStep == cardIndex and self.currentStep ~= nextCardIndex then
    self:scrollToCard(nextCardIndex)
  end
  AHColorTo.new(card.base, duration, ease, color):start()
  AHOpacityTo.new(card.text, duration, ease, 0.5):start()
end

function ClueDisplayCarousel:getCardsOpacityAnimation(duration, opacity)
  local animation = AHSpawn.new()
  local ease = nil

  for _, card in ipairs(self.cards) do
    animation:addAction(AHOpacityTo.new(card.base, duration, ease, opacity))
    animation:addAction(AHOpacityTo.new(card.text, duration, ease, opacity))
  end

  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createCarousel(self)
  local carousel = SphinxCarousel.newClueDisplay(self:getWidth(), self:getHeight())
  self:insertAtCenter(carousel)
  self.carousel = carousel
end

local function makeCardsAndSetupCarousel(self, cluesText)
  local cards = {}
  for _, clueText in ipairs(cluesText) do
    local card = makeCard(self, clueText)
    self.carousel:addCard(card)
    table.insert(cards, card)
  end
  self.carousel:setHorizontalScrollCallback(getOnScrollEndedCallback(self))
  self.cards = cards
  self.currentStep = 1
  self.scrollTargetStep = nil
end

-- Constructor ------------------------

function ClueDisplayCarousel.new(width, height, cluesText)
  local self = Group.new(width, height)
  Mixin.mix(self, ClueDisplayCarousel)

  self.stepChanged = Event.new()

  createCarousel(self)
  makeCardsAndSetupCarousel(self, cluesText)

  self:setTouchEnabled(false)

  return self
end
