--- CooldownGenericButton

class("CooldownGenericButton")

require("sphinx.SharedConstants")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COOLDOWN_DURATION = SharedConstants.BUTTON_COOLDOWN
local COOLDOWN_MASK_OPACITY = SharedConstants.COOLDOWN_MASK_OPACITY

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function reset(self)
  local shineAnim = AHSequence.new()
  local duration = 0.2
  local ease = nil
  shineAnim:addAction(AHOpacityTo.new(self.shine, duration, ease, 0.5))
  shineAnim:addAction(AHOpacityTo.new(self.shine, duration, ease, 0))
  self.updateAction:stop()
  self.button:setTouchEnabled(self.shouldEnableTouchOnNextReset)
  shineAnim:start()
end

local function setScissorRectangle(self, percentToCover)
  local leftX, bottomY = self:getWorldLoc()
  local topY = bottomY + self:getHeight()
  local rightX = leftX + self:getWidth()
  -- to account for rounding
  leftX = leftX - 1
  rightX = rightX + 1
  bottomY = bottomY - 1
  topY = topY + 1
  local scissorBottomY = MathUtils.interpolateLinearFromNormalized(percentToCover, bottomY, topY)
  self.cooldownScissorRectangle:setRect(leftX, topY, rightX, scissorBottomY)
end

local function updateCooldown(self, percentOfCooldownElapsed)
  setScissorRectangle(self, percentOfCooldownElapsed)
  if (percentOfCooldownElapsed >= 1) then
    self.cooldownMask:setVisible(false)
    reset(self)
  end
end

local function getOnPressAction(self, action)
  return function()
    action(self)
    self.cooldownMask:setVisible(true)
    self.updateAction:start()
    self.button:setTouchEnabled(false)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CooldownGenericButton:setCooldownColor(color)
  color = ColorUtils.colorWithOpacity(color, COOLDOWN_MASK_OPACITY)
  self.cooldownMask:setColor(unpack(color))
end

function CooldownGenericButton:setButtonColor(color)
  self.button:setButtonColor(color)
end

function CooldownGenericButton:setCountColor(countColor)
  self.button:setCountColor(countColor)
end

function CooldownGenericButton:initializeCount(initialCount, countColor)
  self.button:initializeCount(initialCount, countColor)
end

function CooldownGenericButton:setCount(count)
  self.button:setCount(count)
end

function CooldownGenericButton:setTouchEnabled(isTouchEnabled)
  self.shouldEnableTouchOnNextReset = isTouchEnabled
  if not self.updateAction:isActive() then
    self.button:setTouchEnabled(isTouchEnabled)
  end
end

function CooldownGenericButton:setVisible(isVisible)
  self.button:setVisible(isVisible)
end

function CooldownGenericButton:setActive(active)
  self.button:setActive(active)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CooldownGenericButton.new(size, iconName, action, buttonColor, cooldownColor)
  cooldownColor = cooldownColor or {1, 1, 1}

  local self = Group.new(size, size)
  Mixin.mix(self, CooldownGenericButton)

  local button = GenericButton.newWithSpriteLabel(iconName, size, buttonColor, getOnPressAction(self, action))
  self:insertAtCenter(button)
  self.button = button

  cooldownColor = ColorUtils.colorWithOpacity(cooldownColor, COOLDOWN_MASK_OPACITY)
  local cooldownMask = FilledCircle.new(size/2, cooldownColor)
  self:insertAtCenter(cooldownMask)
  self.cooldownMask = cooldownMask

  local cooldownScissorRectangle = MOAIScissorRect.new()
  cooldownMask:setScissorRect(cooldownScissorRectangle)
  self.cooldownScissorRectangle = cooldownScissorRectangle

  local shine = FilledCircle.new(size/2, {1,1,1,1})
  shine:setOpacity(0)
  self:insertAtCenter(shine)
  self.shine = shine

  self.updateAction = AHCustomAction.new(COOLDOWN_DURATION, nil, function(normalizedTime)
    updateCooldown(self, normalizedTime)
  end)

  self.shouldEnableTouchOnNextReset = true

  reset(self)

  return self
end
