--- NiceJobDecal

class("NiceJobDecal")

require("sphinx.SphinxColors")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HEIGHT_FRACTION = 0.4

local BLACK = {0,0,0}
local TEAL = {0.0, 0.64, 0.65}

local SIZE_WIDTH_FRACTION = 0.6
local SIZE_HEIGHT_FRACTION = 0.4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function NiceJobDecal:getAnimation()
  local duration = 0.3
  local ease = AHEaseLinear.new()

  local spin = AHRotationTo.new(self, duration, ease, 0)
  local scale = AHScaleTo.new(self, duration, ease, 1)
  local fadeIn = AHSpawn.new(
    AHOpacityTo.new(self.shadow, duration, ease, 0.2),
    AHOpacityTo.new(self.depth, duration, ease, 1),
    AHOpacityTo.new(self.faceplate, duration, ease, 1),
    AHOpacityTo.new(self.label, duration, ease, 1)
  )

  return AHSpawn.new(spin, scale, fadeIn)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function getDecalDimensions()
  local widthSize = Screen.width * SIZE_WIDTH_FRACTION
  local heightSize = Screen.height * SIZE_HEIGHT_FRACTION
  local width = math.min(widthSize, heightSize)
  local height = width * HEIGHT_FRACTION

  return width, height
end

-- Constructor ------------------------

function NiceJobDecal.new(roundNumber)
  local width, height = getDecalDimensions()
  local self = Group.new(width, height)
  Mixin.mix(self, NiceJobDecal)

  local colors = SphinxColors.getFullColorTable()
  local rotation = 6.42

  local shadow = FilledRoundedRectangle.new(width, height, height/2, BLACK)
  self:insertAtCenter(shadow)
  shadow:offsetX(-(width/2) * 0.08)
  shadow:offsetY(-(width/2) * 0.12)
  shadow:setRot(rotation)
  shadow:setOpacity(0)

  local depth = FilledRoundedRectangle.new(width, height, height/2, BLACK)
  self:insertAtCenter(depth)
  depth:setColor(unpack(colors[roundNumber].gameBackground))
  depth:setRot(rotation)
  depth:setOpacity(0)

  local faceplate = FilledRoundedRectangle.new(width, height, height/2, TEAL)
  self:insertAtCenter(faceplate)
  faceplate:setColor(unpack(colors[roundNumber].niceJob))
  faceplate:setOpacity(0)

  local text = "Nice job!"
  local fontSize = height * 0.41
  local font = PrerenderedFont.new(FontFaces.BOLD, fontSize, text)
  local label = Label.new(text, font)
  self:insertAtCenter(label)
  label:setOpacity(0)

  self:setRot(-90)
  self:setScl(0.2)

  self.shadow = shadow
  self.depth = depth
  self.faceplate = faceplate
  self.label = label

  return self
end
