--- ProgressMarker

class("ProgressMarker")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR = {
  WHITE = {1, 1, 1},
  BLACK = {0, 0, 0}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ProgressMarker:setCurrent()
    local fillOpacity = self.fill:getOpacity()
    self.fill:setColor(unpack(COLOR.WHITE))
    self.fill:setOpacity(fillOpacity)
    local labelOpacity = self.label:getOpacity()
    self.label:setColor(unpack(COLOR.BLACK))
    self.label:setOpacity(labelOpacity)
    self.maxOpacity = 1
  end

  function ProgressMarker:setComplete()
    self:setCurrent()
  end

  function ProgressMarker:getHideAnimation(duration)
    return AHSpawn.new(
      AHOpacityTo.new(self.label, duration, nil, 0),
      AHOpacityTo.new(self.fill, duration, nil, 0)
    )
  end

  function ProgressMarker:getShowAnimation(duration)
    return AHSpawn.new(
      AHOpacityTo.new(self.label, duration, nil, 1),
      AHOpacityTo.new(self.fill, duration, nil, self.maxOpacity)
    )
  end

  function ProgressMarker:hide()
    self.label:setOpacity(0)
    self.fill:setOpacity(0)
  end

  function ProgressMarker:show()
    self.label:setOpacity(1)
    self.fill:setOpacity(self.maxOpacity)
  end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ProgressMarker.new(radius, roundNumber)
  local self = Group.new(radius * 2, radius * 2)
  Mixin.mix(self, ProgressMarker)

  self.fill = FilledCircle.new(radius, COLOR.WHITE)
  self.label = Label.new(tostring(roundNumber), Font.new(FontFaces.BOLD, 18))
  self.label:setColor(unpack(COLOR.WHITE))
  self:insertAtCenter(self.fill)
  self:insertAtCenter(self.label)

  self.maxOpacity = 0.2

  return self
end
