--- ScoreTracker

class("ScoreTracker")

require("Event")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Notify game events -----------------

function ScoreTracker:notifyMoveAction()
  self.moveCounts[self.roundNumber] = self.moveCounts[self.roundNumber] + 1
end

function ScoreTracker:notifyUndoAction()

end

function ScoreTracker:notifyRedoAction()

end

function ScoreTracker:notifyRestartAction()

end

function ScoreTracker:setMinimumMoveCount(minimumMoveCount)
  assert(minimumMoveCount > 0, "minimumMoveCount must be greater than 0")
  self.minimumMoveCounts[self.roundNumber] = minimumMoveCount
end

function ScoreTracker:notifyHelpRequested()
  self.hintCount = self.hintCount + 1
end

function ScoreTracker:notifyStartRound(roundNumber, minimumMoveCount)
  assert(self.roundNumber ~= roundNumber, "can't start the same round twice")
  assert(minimumMoveCount > 0, "minimumMoveCount must be greater than 0")
  self.roundNumber = roundNumber
  self.minimumMoveCounts[roundNumber] = minimumMoveCount
end

function ScoreTracker:notifyRoundIsCompleted()
  assert(not self.completedRounds[self.roundNumber], "round is already completed")
  self.completedRounds[self.roundNumber] = true
end

function ScoreTracker:notifyGameIsCompleted()
  assert(self.isGameCompleted == false, "game is already completed")
  self.isGameCompleted = true
end

function ScoreTracker:setTimeElapsed(timeElapsed)
  self.timeElapsed = timeElapsed
end

-- Getters ----------------------------

function ScoreTracker:getGameStats()
  local stats = {
    GameResultScreen.getXPEarned(),
    GameResultScreen.getHintCountStat(self.hintCount)
  }
  return stats
end

function ScoreTracker:getTimeElapsed()
  return self.timeElapsed
end

-- Getters for unit tests -------------

function ScoreTracker:getRoundNumber()
  return self.roundNumber
end

function ScoreTracker:getMoveCounts()
  return self.moveCounts
end

function ScoreTracker:getMinimumMoveCounts()
  return self.minimumMoveCounts
end

function ScoreTracker:getHintCount()
  return self.hintCount
end

function ScoreTracker:getCompletedRounds()
  return self.completedRounds
end

function ScoreTracker:getIsGameCompleted()
  return self.isGameCompleted
end

-- ----------------------------------------------------------------------------
-- Protected API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ScoreTracker.new()
  local self = {}
  Mixin.mix(self, ScoreTracker)

  self.moveCounts = {0, 0, 0}
  self.minimumMoveCounts = {}

  self.hintCount = 0
  self.roundNumber = nil

  self.completedRounds = {false, false, false}
  self.isGameCompleted = false

  self.timeElapsed = 0

  return self
end
