--- SphinxTimer
-- This is a copy of CrossTimer. If the designs do not diverge, we should unify the files.

class("SphinxTimer")

require("sphinx.SphinxTimeDisplay")
require("Timer")
require("sphinx.SphinxColors")
require("sphinx.DropShadowUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH_TO_HEIGHT_RATIO = 100/40

local FONT_SIZE = 24
local FONT = FontFaces.DIN_OT_MEDIUM
local TIME_CHARACTERS = "0123456789:"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Only auto save once every second
local hasWarned = false
local function saveStateIfNeeded(self)
  if (self.saveManager ~= nil) then
    local currentTimeInSeconds = math.floor(self:getTime())
    if (currentTimeInSeconds > self.lastAutoSaveTime) then
      self.saveManager:save()
      self.lastAutoSaveTime = currentTimeInSeconds
    end
  elseif not hasWarned then
    Log.warn("No save manager has been set - not saving progress.")
    hasWarned = true
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SphinxTimer:start()
  self.timeTracker:start()
end

function SphinxTimer:pause()
  self.timeTracker:pause()
end

function SphinxTimer:getTime()
  return self.timeTracker:getTime()
end

function SphinxTimer:setInitialTime(time)
  -- Update the last auto save time so the timer will not
  -- trigger a save when setTime() is called
  self.lastAutoSaveTime = time
  self.timeTracker:setTime(time)
end

function SphinxTimer:reset()
  self.timeTracker:setTime(0)
end

function SphinxTimer:setSaveManager(saveManager)
  self.saveManager = saveManager
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createBackground(self, color)
  local width = self:getWidth()
  local height = self:getHeight()
  local cornerRadius = height/2
  local background = FilledRoundedRectangle.new(width, height, cornerRadius, color)
  self:insertAtCenter(background)
end

local function createDropShadow(self)
  local width = self:getWidth()
  local height = self:getHeight()
  local cornerRadius = height/2
  local dropShadow = FilledRoundedRectangle.new(width, height, cornerRadius, SphinxColors.DROP_SHADOW)
  self:insertAtCenter(dropShadow)

  DropShadowUtils.offsetDropShadow(dropShadow)
end

-- Constructor ------------------------

function SphinxTimer.new(height, color)
  local width = height * WIDTH_TO_HEIGHT_RATIO
  local self = Group.new(width, height)
  Mixin.mix(self, SphinxTimer)

  createDropShadow(self)
  createBackground(self, color)

  local prerenderedFont = PrerenderedFont.newWithActualSize(FONT, FONT_SIZE, TIME_CHARACTERS)
  self.timeDisplay = SphinxTimeDisplay.new(prerenderedFont)
  FunctionUtils.forwardMethodCallsInClass(self, self.timeDisplay, SphinxTimeDisplay)
  self:insertAtCenter(self.timeDisplay)

  self.timeTracker = {}
  Timer.makeTimer(self.timeTracker, math.huge)
  self.timeTracker:initialize()

  self.lastAutoSaveTime = 0
  self.timeTracker:setUpdateHandler(function()
    saveStateIfNeeded(self)
    self:setDisplayedTime(self:getTime())
  end)

  return self
end
