--- RiverBendDragTracker

class("RiverBendDragTracker")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiverBendDragTracker:isCellAdjacentToLast(rowIndex, columnIndex)
  if #self.draggedWordIndices == 0 then
    return true
  end

  local previousCellRowIndex, previousCellColumnIndex = unpack(self.draggedWordIndices[#self.draggedWordIndices])
  local rowDiff = math.abs(rowIndex - previousCellRowIndex)
  local colDiff = math.abs(columnIndex - previousCellColumnIndex)

  return rowDiff + colDiff == 1
end

function RiverBendDragTracker:doesCellExceedLimit(rowIndex, columnIndex)
  assert(#self.draggedWordIndices <= 12, "Word size cannot exceed 12")
  if #self.draggedWordIndices == 12 then
    for index, draggedCellIndices in ipairs(self.draggedWordIndices) do
      if TableUtils.areArraysEqual({rowIndex, columnIndex}, draggedCellIndices) then
        return false
      end
    end
    return true
  else
    return false
  end
end

function RiverBendDragTracker:isDraggedWordEmpty()
  return #self.draggedWordIndices == 0
end

function RiverBendDragTracker:updateLastSelectedCell(rowIndex, columnIndex)
  local removedLettersIndicesList = {}
  local addedLetterIndices = {rowIndex, columnIndex}

  for index, draggedCellIndices in ipairs(self.draggedWordIndices) do
    if TableUtils.areArraysEqual({rowIndex, columnIndex}, draggedCellIndices) then
      addedLetterIndices = nil

      for i = #self.draggedWordIndices, index + 1, -1 do
        table.insert(removedLettersIndicesList, self.draggedWordIndices[i])
        table.remove(self.draggedWordIndices)
      end

      break
    end
  end

  if addedLetterIndices ~= nil then
    table.insert(self.draggedWordIndices, addedLetterIndices)
  end

  self.lengthChanged:call(removedLettersIndicesList, addedLetterIndices)
end

function RiverBendDragTracker:resetSelection()
  self.draggedWordIndices = {}
end

function RiverBendDragTracker:getDraggedWordLength()
  return #self.draggedWordIndices
end

function RiverBendDragTracker:getDraggedCellIndices(cellIndex)
  return self.draggedWordIndices[cellIndex]
end

-- ----------------------------------------------------------------------------
-- Tests API
-- ----------------------------------------------------------------------------

function RiverBendDragTracker:getDraggedWordIndices()
  return self.draggedWordIndices
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RiverBendDragTracker.new()
  local self = {}
  Mixin.mix(self, RiverBendDragTracker)

  self.draggedWordIndices = {}

  self.lengthChanged = Event.new()

  return self
end
