--- RiverBendHintHelper

class("RiverBendHintHelper")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getNextClueIndex(self, clueIndex)
  assert(not self.model:isSolved(), "cannot update index if model is solved")
  clueIndex = clueIndex % self.model:getClueCount() + 1
  if self.model:isClueCompleted(clueIndex) then
    return getNextClueIndex(self, clueIndex)
  else
    return clueIndex
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiverBendHintHelper:doHint()
  local clueDisplayIndex = self.clueDisplay:getCurrentStep()
  if self.model:isClueCompleted(clueDisplayIndex) then
    clueDisplayIndex = getNextClueIndex(self, clueDisplayIndex)
  end

  local clueAnswerLength = self.model:getClueAnswerLength(clueDisplayIndex)
  local widthInCells, heightInCells = self.model:getBoardDimensions()

  for rowIndex = 1, heightInCells do
    for columnIndex = 1, widthInCells do
      local cellClueIndex = self.model:getCellClueIndex(rowIndex, columnIndex)
      local cellAnswerIndex = self.model:getCellAnswerIndex(rowIndex, columnIndex)

      if cellClueIndex == clueDisplayIndex and cellAnswerIndex == self.hintIndex then
        self.hintIndex = self.hintIndex + 1
        self.grid:revealCellHighlight(rowIndex, columnIndex)

        if self.hintIndex > clueAnswerLength then
          self.hintButton:setActive(false)
        end

        return
      end
    end
  end
end

function RiverBendHintHelper:resetHintIndex()
  self.hintIndex = 1
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RiverBendHintHelper.new(model, grid, clueDisplay, hintButton)
  local self = {}
  Mixin.mix(self, RiverBendHintHelper)

  self.model = model
  self.grid = grid
  self.clueDisplay = clueDisplay
  self.hintButton = hintButton
  self.hintIndex = 1

  return self
end
