--- RiverBendInput
-- @author Created by andrewcousins on November 12, 2024.

class("RiverBendInput")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function canCellBeAdded(self, rowIndex, columnIndex)
  local cellState = self.model:getCellState(rowIndex, columnIndex)
  local isAdjacentToLast = self.dragTracker:isCellAdjacentToLast(rowIndex, columnIndex)
  local doesExceedLimit = self.dragTracker:doesCellExceedLimit(rowIndex, columnIndex)
  return isAdjacentToLast and (cellState == RiverBendCellStates.ACTIVE or (cellState == RiverBendCellStates.INACTIVE and not doesExceedLimit))
end

local function touchBegan(self, x, y)
  if self.grid:inside(x, y) then
    local rowIndex, columnIndex = self.grid:getCellIndicesAt(x, y)
    if self.model:getCellState(rowIndex, columnIndex) ~= RiverBendCellStates.COMPLETE then
      self.dragTracker:updateLastSelectedCell(rowIndex, columnIndex)
    end
  end
end

local function touchMoved(self, x, y)
  if self.grid:inside(x, y) then
    local rowIndex, columnIndex = self.grid:getCellIndicesAt(x, y)
    if canCellBeAdded(self, rowIndex, columnIndex) then
      self.dragTracker:updateLastSelectedCell(rowIndex, columnIndex)
    end
  end
end

local function touchEnded(self)
  self.wordSubmitted:call()
end

local function getTouchHandler(self)
  return function(event)
    if (event.phase == TouchManager.TOUCH_BEGAN) then
      return touchBegan(self, event.x, event.y)
    elseif (event.phase == TouchManager.TOUCH_MOVED) then
        return touchMoved(self, event.x, event.y)
    elseif (event.phase == TouchManager.TOUCH_ENDED) then
      return touchEnded(self)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RiverBendInput.new(model, grid, dragTracker)
  local self = Group.newFullScreenSafe()
  Mixin.mix(self, RiverBendInput)

  self.model = model
  self.grid = grid
  self.dragTracker = dragTracker

  self.wordSubmitted = Event.new()

  TouchManager.addTouchEnabledMethods(self, getTouchHandler(self))

  return self
end
