--- RiverBendInterstitialShapes

class("RiverBendInterstitialShapes")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WEATHER_DURATION_DEFAULT = 3
local SUN_DELAY = 0.25
local TERRAIN_DURATION = 0.5
local RIVER_DURATION = 1
local RIVER_DELAY = 0.75
local TREE_DURATION = 0.5
local BACKGROUND_TREE_DELAY = 0.5
local FOREGROUND_TREE_DELAY = 1.5
local HOUSE_DURATION = 1
local HOUSE_DELAY = 1
local SMOKE_DURATION = 0.5
local SMOKE_DELAY = 1.5
local DEER_DURATION = 1
local DEER_DELAY = 1.75
local OUTRO_DURATION = 0.75

local SPRITE_NAMES = {
  CLOUD_LEFT = "cloud_left.png",
  CLOUD_MIDDLE = "cloud_middle.png",
  CLOUD_RIGHT = "cloud_right.png",
  SUN = "sun.png",
  TERRAIN_BACK = "terrain_back.png",
  TERRAIN_FRONT = "terrain_front.png",
  RIVER = "river.png",
  TREE_BACKGROUND = "tree_background.png",
  TREE_FOREGROUND = "tree_foreground_%d.png",
  HOUSE_LAYER = "house_layer_%d.png",
  HOUSE_WINDOW_OFF = "house_window_off.png",
  HOUSE_WINDOW_ON = "house_window_on.png",
  SMOKE = "smoke_%d.png",
  DEER = "deer.png"
}

local Z_INDEX_INTERSTITIAL = {
  CLOUDS = 0,
  SUN = 10,
  TERRAIN_BACK = 20,
  RIVER = 30,
  TERRAIN_FRONT = 40,
  TREES_BACKGROUND = 50,
  HOUSE = 60,
  SMOKE = 70,
  DEER = 80,
  TREES_FOREGROUND = 90
}

local Z_INDEX_RIVER = {
  WATER = 0,
  MASK = 10
}

local Z_INDEX_HOUSE = {
  SHADOW = 0,
  BODY = 10,
  ROOF = 20,
  GUTTER = 30,
  WINDOW = 40
}

local CLOUD_DATA = {
  {
    imagePath = SPRITE_NAMES.CLOUD_MIDDLE,
    initialScale = 5.0,
    finalScale = 3.0,
    initialXPart = 0.53,
    finalXPart = 0.53,
    initialYPart = 0.85,
    finalYPart = 0.93
  },
  {
    imagePath = SPRITE_NAMES.CLOUD_LEFT,
    initialScale = 6.0,
    finalScale = 4.2,
    initialXPart = 0.2,
    finalXPart = 0.15,
    initialYPart = 0.75,
    finalYPart = 0.85
  },
  {
    imagePath = SPRITE_NAMES.CLOUD_RIGHT,
    initialScale = 6.0,
    finalScale = 4.2,
    initialXPart = 0.75,
    finalXPart = 0.80,
    initialYPart = 0.66,
    finalYPart = 0.75
  }
}

local SUN_DATA = {
  scale = 2.5,
  xPart = 0.545,
  initialYPart = 0.35,
  finalYPart = 0.532
}

local TERRAIN_DATA = {
  {
    scale = 2.1,
    initialXPart = 0.6,
    finalXPart = 0.6,
    initialYPart = -0.21,
    finalYPart = 0.58
  },
  {
    scale = 1.95,
    initialXPart = -0.7,
    finalXPart = 0.5675,
    initialYPart = 0.475,
    finalYPart = 0.475
  }
}

local RIVER_DATA = {
  scale = 1.95,
  xPart = 0.67,
  yPart = 0.495,
  maskOffset = -0.13
}

local BACKGROUND_TREE_DATA = {
  {scale = 0.59, xPart = 0.198, yPart = 0.497},
  {scale = 0.47, xPart = 0.915, yPart = 0.492},
  {scale = 0.32, xPart = 0.800, yPart = 0.485},
  {scale = 0.44, xPart = 0.700, yPart = 0.440},
  {scale = 0.68, xPart = 0.305, yPart = 0.440},
  {scale = 0.33, xPart = 0.650, yPart = 0.432},
  {scale = 0.32, xPart = 0.060, yPart = 0.400},
  {scale = 0.74, xPart = 0.475, yPart = 0.390},
  {scale = 0.99, xPart = 0.178, yPart = 0.369},
  {scale = 0.70, xPart = 0.380, yPart = 0.375},
  {scale = 0.72, xPart = 0.260, yPart = 0.354},
  {scale = 0.98, xPart = 0.765, yPart = 0.360},
  {scale = 1.15, xPart = 0.870, yPart = 0.357},
  {scale = 0.96, xPart = 0.950, yPart = 0.335}
}

local FOREGROUND_TREE_DATA = {
  {
    initialScale = 1.3,
    finalScale = 0.82,
    xPart = 0.15,
    yPart = 0.29
  },
  {
    initialScale = 1.9,
    finalScale = 1.2,
    xPart = 0.93,
    yPart = 0.28
  }
}

local HOUSE_DATA = {
  scale = 2.23,
  initialXPart = 0.85,
  finalXPart = 0.73,
  yPart = 0.375
}

local SMOKE_DATA = {
  {
    initialScale = 0.5,
    finalScale = 1.75,
    initialXPart = 0.746,
    finalXPart = 0.715,
    initialYPart = 0.415,
    finalYPart = 0.438,
    riseDuration = 2
  },
  {
    initialScale = 0.5,
    finalScale = 1.75,
    initialXPart = 0.746,
    finalXPart = 0.765,
    initialYPart = 0.415,
    finalYPart = 0.43,
    riseDuration = 1.5
  },
  {
    initialScale = 0.5,
    finalScale = 1.75,
    initialXPart = 0.746,
    finalXPart = 0.735,
    initialYPart = 0.415,
    finalYPart = 0.42,
    riseDuration = 1
  }
}

local DEER_DATA = {
  scale = 1.15,
  initialXPart = -0.3,
  finalXPart = 0.27,
  yPart = 0.295
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWeatherDuration(self, roundNumber)
  local weatherDuration = WEATHER_DURATION_DEFAULT
  if roundNumber >= 2 then
    weatherDuration = math.max(weatherDuration, SMOKE_DATA[1].riseDuration + SMOKE_DELAY)
  end

  if roundNumber == 3 then
    weatherDuration = math.max(weatherDuration, DEER_DURATION + DEER_DELAY)
  end

  return weatherDuration
end

local function getSmokeAnimation(self)
  local animation = AHStaggeredSpawn.new(0.5)
  local duration = SMOKE_DURATION

  for index, smokeCloud in ipairs(self.smoke) do
    local smokeCloudData = SMOKE_DATA[index]
    local smokeCloudAnimation = AHSpawn.new()
    smokeCloudAnimation:addAction(AHOpacityTo.new(smokeCloud, duration, nil, 1))
    smokeCloudAnimation:addAction(AHScaleTo.new(smokeCloud, duration, nil, smokeCloudData.finalScale))
    smokeCloudAnimation:addAction(AHLocationTo.new(smokeCloud, smokeCloudData.riseDuration, nil, self:getWidthPart(smokeCloudData.finalXPart), self:getHeightPart(smokeCloudData.finalYPart)))
    animation:addAction(smokeCloudAnimation)
  end

  return animation
end

local function makeIntroAnimation(self, roundNumber)
  local cloudsAnimation = AHStaggeredSpawn.new(0.1)
  local sunAnimation = AHSpawn.new()
  local terrainsAnimation = AHStaggeredSpawn.new(0.2)
  local riverAnimation = AHSpawn.new()
  local backgroundTreesAnimation = AHStaggeredSpawn.new(0.005)
  local foregroundTreesAnimation = AHStaggeredSpawn.new(0.1)
  local houseAnimation = AHSpawn.new()
  local smokeAnimation = AHSpawn.new()
  local deerAnimation = AHSpawn.new()
  local weatherDuration = getWeatherDuration(self, roundNumber)
  local sunDuration = weatherDuration - SUN_DELAY
  local ease = AHEaseQuadraticOut.new()

  for index, cloud in ipairs(self.clouds) do
    local cloudData = CLOUD_DATA[index]
    local cloudAnimation = AHSpawn.new()
    cloudAnimation:addAction(AHScaleTo.new(cloud, weatherDuration, ease, cloudData.finalScale))
    cloudAnimation:addAction(AHLocationTo.new(cloud, weatherDuration, ease, self:getWidthPart(cloudData.finalXPart), self:getHeightPart(cloudData.finalYPart)))
    cloudAnimation:addAction(AHOpacityTo.new(cloud, weatherDuration, ease, 1))
    cloudsAnimation:addAction(cloudAnimation)
  end

  sunAnimation:addAction(AHLocationYTo.new(self.sun, sunDuration, AHEaseQuadraticOut.new(), self:getHeightPart(SUN_DATA.finalYPart)))
  sunAnimation:addAction(AHOpacityTo.new(self.sun, sunDuration, ease, 1))

  for index, terrain in ipairs(self.terrains) do
    local terrainData = TERRAIN_DATA[index]
    local terrainAnimation = AHSpawn.new()
    terrainAnimation:addAction(function() terrain:setOpacity(1) end)
    terrainAnimation:addAction(AHLocationTo.new(terrain, TERRAIN_DURATION, ease, self:getWidthPart(terrainData.finalXPart), self:getHeightPart(terrainData.finalYPart)))
    terrainsAnimation:addAction(terrainAnimation)
  end

  riverAnimation:addAction(function() self.river:setOpacity(1) end)
  riverAnimation:addAction(AHLocationYBy.new(self.river.mask, RIVER_DURATION, nil, -self.river:getHeight()))

  for index, backgroundTree in ipairs(self.backgroundTrees) do
    local backgroundTreeData = BACKGROUND_TREE_DATA[index]
    backgroundTreesAnimation:addAction(AHScaleTo.new(backgroundTree, TREE_DURATION, ease, backgroundTreeData.scale))
  end

  for index, foregroundTree in ipairs(self.foregroundTrees) do
    local foregroundTreeData = FOREGROUND_TREE_DATA[index]
    foregroundTreesAnimation:addAction(AHScaleTo.new(foregroundTree, TREE_DURATION, ease, foregroundTreeData.finalScale))
    foregroundTreesAnimation:addAction(AHOpacityTo.new(foregroundTree, TREE_DURATION, ease, 1))
  end

  houseAnimation:addAction(AHLocationXTo.new(self.house, HOUSE_DURATION, ease, self:getWidthPart(HOUSE_DATA.finalXPart)))
  houseAnimation:addAction(AHOpacityTo.new(self.house, HOUSE_DURATION, ease, 1))

  if roundNumber >= 2 then
    smokeAnimation:addAction(getSmokeAnimation(self))
  end

  if roundNumber == 3 then
    deerAnimation:addAction(function() self.deer:setOpacity(1) end)
    deerAnimation:addAction(AHLocationXTo.new(self.deer, DEER_DURATION, ease, self:getWidthPart(DEER_DATA.finalXPart)))
  end

  return AHSpawn.new(cloudsAnimation,
                     AHDelayed.new(SUN_DELAY, sunAnimation),
                     terrainsAnimation,
                     AHDelayed.new(RIVER_DELAY, riverAnimation),
                     AHDelayed.new(BACKGROUND_TREE_DELAY, backgroundTreesAnimation),
                     AHDelayed.new(FOREGROUND_TREE_DELAY, foregroundTreesAnimation),
                     AHDelayed.new(HOUSE_DELAY, houseAnimation),
                     AHDelayed.new(SMOKE_DELAY, smokeAnimation),
                     AHDelayed.new(DEER_DELAY, deerAnimation))
end

local function makeOutroAnimation(self)
  local animation = AHSpawn.new()
  animation:addAction(AHOpacityTo.new(self.sun, OUTRO_DURATION, nil, 0))
  animation:addAction(AHOpacityTo.new(self.river, OUTRO_DURATION, nil, 0))

  return animation
end

local function createClouds(self)
  self.clouds = {}

  for _, cloudData in ipairs(CLOUD_DATA) do
    local cloud = Sprite.newWithName(cloudData.imagePath)
    cloud:setColor(unpack(SphinxColors.getRoundColor("mid")))
    cloud:setScl(cloudData.initialScale)
    cloud:setOpacity(0)
    self:insertAtLocationFractional(cloud, cloudData.initialXPart, cloudData.initialYPart, Z_INDEX_INTERSTITIAL.CLOUDS)
    table.insert(self.clouds, cloud)
  end
end

local function createSun(self)
  local sunData = SUN_DATA
  local sun = Sprite.newWithName(SPRITE_NAMES.SUN)
  sun:setScl(sunData.scale)
  sun:setOpacity(0)
  self:insertAtLocationFractional(sun, sunData.xPart, sunData.initialYPart, Z_INDEX_INTERSTITIAL.SUN)

  self.sun = sun
end

local function createTerrains(self)
  self.terrains = {}

  local terrainBackData = TERRAIN_DATA[1]
  local terrainBack = Sprite.newWithName(SPRITE_NAMES.TERRAIN_BACK)
  terrainBack:setColor(unpack(SphinxColors.getRoundColor("mid")))
  terrainBack:setScl(terrainBackData.scale)
  terrainBack:setAnchorPointTop()
  terrainBack:setOpacity(0)
  self:insertAtLocationFractional(terrainBack, terrainBackData.initialXPart, terrainBackData.initialYPart, Z_INDEX_INTERSTITIAL.TERRAIN_BACK)
  table.insert(self.terrains, terrainBack)

  local terrainFrontData = TERRAIN_DATA[2]
  local terrainFront = Sprite.newWithName(SPRITE_NAMES.TERRAIN_FRONT)
  terrainFront:setColor(unpack(SphinxColors.getRoundColor("mid")))
  terrainFront:setScl(terrainFrontData.scale)
  terrainFront:setAnchorPointRight()
  terrainFront:setOpacity(0)
  self:insertAtLocationFractional(terrainFront, terrainFrontData.initialXPart, terrainFrontData.initialYPart, Z_INDEX_INTERSTITIAL.TERRAIN_FRONT)
  table.insert(self.terrains, terrainFront)
end

local function createRiver(self)
  local water = Sprite.newWithName(SPRITE_NAMES.RIVER)
  local color = SphinxColors.getRoundColor("mid")
  local mask = FilledRectangle.new(water:getWidth(), water:getHeight(), color)
  local river = Group.new(water:getWidth(), water:getHeight())
  river:insertAtCenter(water, Z_INDEX_RIVER.WATER)
  river:insertAtCenter(mask, Z_INDEX_RIVER.MASK)
  river.mask = mask

  river:linkOpacityAttrToChildren()
  river:setScl(RIVER_DATA.scale)
  river:setAnchorPointTop()
  river:setOpacity(0)
  self:insertAtLocationFractional(river, RIVER_DATA.xPart, RIVER_DATA.yPart, Z_INDEX_INTERSTITIAL.RIVER)
  river.mask:offsetY(river:getHeightPart(RIVER_DATA.maskOffset))

  self.river = river
end

local function createTrees(self, roundNumber)
  self.backgroundTrees = {}
  self.foregroundTrees = {}

  for _, backgroundTreeData in ipairs(BACKGROUND_TREE_DATA) do
    local backgroundTree = Sprite.newWithName(SPRITE_NAMES.TREE_BACKGROUND)
    backgroundTree:setColor(unpack(SphinxColors.getRoundColor("darkest")))
    backgroundTree:setAnchorPointBottom()
    backgroundTree:setScl(0)
    self:insertAtLocationFractional(backgroundTree, backgroundTreeData.xPart, backgroundTreeData.yPart, Z_INDEX_INTERSTITIAL.TREES_BACKGROUND)
    table.insert(self.backgroundTrees, backgroundTree)
  end

  for _, foregroundTreeData in ipairs(FOREGROUND_TREE_DATA) do
    local foregroundTree = Sprite.newWithName(string.format(SPRITE_NAMES.TREE_FOREGROUND, roundNumber))
    foregroundTree:setScl(foregroundTreeData.initialScale)
    foregroundTree:setOpacity(0)
    self:insertAtLocationFractional(foregroundTree, foregroundTreeData.xPart, foregroundTreeData.yPart, Z_INDEX_INTERSTITIAL.TREES_FOREGROUND)
    table.insert(self.foregroundTrees, foregroundTree)
  end
end

local function createHouseWindowSprite(self, roundNumber)
  local houseWindow = nil
  if roundNumber == 1 then
    houseWindow = Sprite.newWithName(SPRITE_NAMES.HOUSE_WINDOW_OFF)
    houseWindow:setColor(unpack(SphinxColors.getRoundColor("lighter")))
  else
    houseWindow = Sprite.newWithName(SPRITE_NAMES.HOUSE_WINDOW_ON)
  end

  return houseWindow
end

local function createHouse(self, roundNumber)
  local houseShadow = Sprite.newWithName(string.format(SPRITE_NAMES.HOUSE_LAYER, 1))
  houseShadow:setColor(unpack(SphinxColors.getRoundColor("darkest")))
  local houseBody = Sprite.newWithName(string.format(SPRITE_NAMES.HOUSE_LAYER, 2))
  houseBody:setColor(unpack(SphinxColors.getRoundColor("dark")))
  local houseRoof = Sprite.newWithName(string.format(SPRITE_NAMES.HOUSE_LAYER, 3))
  houseRoof:setColor(unpack(SphinxColors.getRoundColor("lighter")))
  local houseGutter = Sprite.newWithName(string.format(SPRITE_NAMES.HOUSE_LAYER, 4))
  houseGutter:setColor(unpack(SphinxColors.getRoundColor("light")))
  local houseWindow = createHouseWindowSprite(self, roundNumber)

  local house = Group.new(houseShadow:getWidth(), houseShadow:getHeight())
  house:insertAtCenter(houseShadow, Z_INDEX_HOUSE.SHADOW)
  house:insertAtCenter(houseBody, Z_INDEX_HOUSE.BODY)
  house:insertAtCenter(houseRoof, Z_INDEX_HOUSE.ROOF)
  house:insertAtCenter(houseGutter, Z_INDEX_HOUSE.GUTTER)
  house:insertAtCenter(houseWindow, Z_INDEX_HOUSE.WINDOW)
  house:linkOpacityAttrToChildren()
  house:setOpacity(0)
  house:setScl(HOUSE_DATA.scale)
  self:insertAtLocationFractional(house, HOUSE_DATA.initialXPart, HOUSE_DATA.yPart, Z_INDEX_INTERSTITIAL.HOUSE)

  self.house = house
end

local function createSmoke(self)
  self.smoke = {}

  for index, smokeCloudData in ipairs(SMOKE_DATA) do
    local smokeCloud = Sprite.newWithName(string.format(SPRITE_NAMES.SMOKE, index))
    smokeCloud:setScl(smokeCloudData.initialScale)
    smokeCloud:setOpacity(0)
    self:insertAtLocationFractional(smokeCloud, smokeCloudData.initialXPart, smokeCloudData.initialYPart, Z_INDEX_INTERSTITIAL.SMOKE)
    table.insert(self.smoke, smokeCloud)
  end
end

local function createDeer(self)
  local deerData = DEER_DATA
  local deer = Sprite.newWithName(SPRITE_NAMES.DEER)
  deer:setColor(unpack(SphinxColors.getRoundColor("darkest")))
  deer:setScl(deerData.scale)
  deer:setOpacity(0)
  self:insertAtLocationFractional(deer, deerData.initialXPart, deerData.yPart, Z_INDEX_INTERSTITIAL.DEER)

  self.deer = deer
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiverBendInterstitialShapes:getIntroAnimation(roundNumber)
  return makeIntroAnimation(self, roundNumber)
end

function RiverBendInterstitialShapes:getOutroAnimation(roundNumber)
  return makeOutroAnimation(self)
end

function RiverBendInterstitialShapes:initialize(roundNumber)
  createClouds(self)
  createSun(self)
  createTerrains(self)
  createRiver(self)
  createTrees(self, roundNumber)
  createHouse(self, roundNumber)
  createSmoke(self)
  createDeer(self)
end

function RiverBendInterstitialShapes:clear()
  self:removeAll()
end

function RiverBendInterstitialShapes:getCurrentScale()
  return 1
end

function RiverBendInterstitialShapes:getHoldOnScreenScaleDuration()
  return 3.5
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RiverBendInterstitialShapes.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, RiverBendInterstitialShapes)

  return self
end
