--- RiverBendLinkTypeCalculator

class("RiverBendLinkTypeCalculator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getEndCellLinkType(rowDiff, colDiff)
  if rowDiff == 1 then
    return RiverBendCellStates.LINK_TYPE.DOWN
  elseif rowDiff == -1 then
    return RiverBendCellStates.LINK_TYPE.UP
  elseif colDiff == 1 then
    return RiverBendCellStates.LINK_TYPE.RIGHT
  elseif colDiff == -1 then
    return RiverBendCellStates.LINK_TYPE.LEFT
  end
end

local function getMiddleCellLinkType(prevRowDiff, prevColDiff, nextRowDiff, nextColDiff)
  if (prevRowDiff == 1 and nextRowDiff == -1) or (prevRowDiff == -1 and nextRowDiff == 1) then
    return RiverBendCellStates.LINK_TYPE.UP_DOWN
  elseif (prevColDiff == 1 and nextColDiff == -1) or (prevColDiff == -1 and nextColDiff == 1) then
    return RiverBendCellStates.LINK_TYPE.LEFT_RIGHT
  elseif (prevRowDiff == 1 and nextColDiff == -1) or (prevColDiff == -1 and nextRowDiff == 1) then
    return RiverBendCellStates.LINK_TYPE.LEFT_DOWN
  elseif (prevRowDiff == 1 and nextColDiff == 1) or (prevColDiff == 1 and nextRowDiff == 1) then
    return RiverBendCellStates.LINK_TYPE.RIGHT_DOWN
  elseif (prevRowDiff == -1 and nextColDiff == 1) or (prevColDiff == 1 and nextRowDiff == -1) then
    return RiverBendCellStates.LINK_TYPE.UP_RIGHT
  elseif (prevRowDiff == -1 and nextColDiff == -1) or (prevColDiff == -1 and nextRowDiff == -1) then
    return RiverBendCellStates.LINK_TYPE.UP_LEFT
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiverBendLinkTypeCalculator.getLinkType(cellIndices, prevCellIndices, nextCellIndices)
  local rowIndex, columnIndex = unpack(cellIndices)
  local prevRowDiff = nil
  local prevColDiff = nil
  local nextRowDiff = nil
  local nextColDiff = nil

  if prevCellIndices then
    local prevCellRowIndex, prevCellColumnIndex = unpack(prevCellIndices)
    prevRowDiff = prevCellRowIndex - rowIndex
    prevColDiff = prevCellColumnIndex - columnIndex
  end

  if nextCellIndices then
    local nextCellRowIndex, nextCellColumnIndex = unpack(nextCellIndices)
    nextRowDiff = nextCellRowIndex - rowIndex
    nextColDiff = nextCellColumnIndex - columnIndex
  end

  if not prevCellIndices and not nextCellIndices then
    return RiverBendCellStates.LINK_TYPE.NONE
  elseif prevCellIndices and not nextCellIndices then
    return getEndCellLinkType(prevRowDiff, prevColDiff)
  elseif not prevCellIndices and nextCellIndices then
    return getEndCellLinkType(nextRowDiff, nextColDiff)
  elseif prevCellIndices and nextCellIndices then
    return getMiddleCellLinkType(prevRowDiff, prevColDiff, nextRowDiff, nextColDiff)
  end
end
